@@ -33,13 +33,14 @@ import net.lingala.zip4j.model.enums.AesKeyStrength;
 import net.lingala.zip4j.model.enums.AesVersion;
 import net.lingala.zip4j.model.enums.CompressionMethod;
 import net.lingala.zip4j.model.enums.EncryptionMethod;
+import net.lingala.zip4j.util.InternalZipConstants;
 import net.lingala.zip4j.util.RawIO;
 
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.RandomAccessFile;
 import java.math.BigInteger;
-import java.nio.charset.StandardCharsets;
+import java.nio.charset.Charset;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
@@ -47,6 +48,7 @@ import java.util.List;
 import static net.lingala.zip4j.headers.HeaderUtil.decodeStringWithCharset;
 import static net.lingala.zip4j.util.BitUtils.isBitSet;
 import static net.lingala.zip4j.util.InternalZipConstants.ENDHDR;
+import static net.lingala.zip4j.util.InternalZipConstants.ZIP_64_SIZE_LIMIT;
 
 /**
  * Helper class to read header information for the zip file
@@ -57,7 +59,7 @@ public class HeaderReader {
   private RawIO rawIO = new RawIO();
   private byte[] intBuff = new byte[4];
 
-  public ZipModel readAllHeaders(RandomAccessFile zip4jRaf) throws IOException {
+  public ZipModel readAllHeaders(RandomAccessFile zip4jRaf, Charset charset) throws IOException {
     zipModel = new ZipModel();
 
     try {
@@ -79,7 +81,7 @@ public class HeaderReader {
       }
     }
 
-    zipModel.setCentralDirectory(readCentralDirectory(zip4jRaf, rawIO));
+    zipModel.setCentralDirectory(readCentralDirectory(zip4jRaf, rawIO, charset));
 
     return zipModel;
   }
@@ -118,7 +120,7 @@ public class HeaderReader {
     if (commentLength > 0) {
       byte[] commentBuf = new byte[commentLength];
       zip4jRaf.readFully(commentBuf);
-      endOfCentralDirectoryRecord.setComment(new String(commentBuf, StandardCharsets.UTF_8));
+      endOfCentralDirectoryRecord.setComment(new String(commentBuf, InternalZipConstants.CHARSET_UTF_8));
     } else {
       endOfCentralDirectoryRecord.setComment(null);
     }
@@ -127,7 +129,7 @@ public class HeaderReader {
     return endOfCentralDirectoryRecord;
   }
 
-  private CentralDirectory readCentralDirectory(RandomAccessFile zip4jRaf, RawIO rawIO) throws IOException {
+  private CentralDirectory readCentralDirectory(RandomAccessFile zip4jRaf, RawIO rawIO, Charset charset) throws IOException {
     CentralDirectory centralDirectory = new CentralDirectory();
     List<FileHeader> fileHeaders = new ArrayList<>();
 
@@ -195,7 +197,7 @@ public class HeaderReader {
       if (fileNameLength > 0) {
         byte[] fileNameBuff = new byte[fileNameLength];
         zip4jRaf.readFully(fileNameBuff);
-        String fileName = decodeStringWithCharset(fileNameBuff, fileHeader.isFileNameUTF8Encoded());
+        String fileName = decodeStringWithCharset(fileNameBuff, fileHeader.isFileNameUTF8Encoded(), charset);
 
         if (fileName.contains(":\\")) {
           fileName = fileName.substring(fileName.indexOf(":\\") + 2);
@@ -214,7 +216,7 @@ public class HeaderReader {
       if (fileCommentLength > 0) {
         byte[] fileCommentBuff = new byte[fileCommentLength];
         zip4jRaf.readFully(fileCommentBuff);
-        fileHeader.setFileComment(decodeStringWithCharset(fileCommentBuff, fileHeader.isFileNameUTF8Encoded()));
+        fileHeader.setFileComment(decodeStringWithCharset(fileCommentBuff, fileHeader.isFileNameUTF8Encoded(), charset));
       }
 
       if (fileHeader.isEncrypted()) {
@@ -410,17 +412,30 @@ public class HeaderReader {
       return;
     }
 
-    Zip64ExtendedInfo zip64ExtendedInfo = readZip64ExtendedInfo(fileHeader.getExtraDataRecords(), rawIO);
+    Zip64ExtendedInfo zip64ExtendedInfo = readZip64ExtendedInfo(fileHeader.getExtraDataRecords(), rawIO,
+        fileHeader.getUncompressedSize(), fileHeader.getCompressedSize(), fileHeader.getOffsetLocalHeader());
 
     if (zip64ExtendedInfo == null) {
       return;
     }
 
     fileHeader.setZip64ExtendedInfo(zip64ExtendedInfo);
-    fileHeader.setUncompressedSize(zip64ExtendedInfo.getUncompressedSize());
-    fileHeader.setCompressedSize(zip64ExtendedInfo.getCompressedSize());
-    fileHeader.setOffsetLocalHeader(zip64ExtendedInfo.getOffsetLocalHeader());
-    fileHeader.setDiskNumberStart(zip64ExtendedInfo.getDiskNumberStart());
+
+    if (zip64ExtendedInfo.getUncompressedSize() != -1) {
+      fileHeader.setUncompressedSize(zip64ExtendedInfo.getUncompressedSize());
+    }
+
+    if (zip64ExtendedInfo.getCompressedSize() != -1) {
+      fileHeader.setCompressedSize(zip64ExtendedInfo.getCompressedSize());
+    }
+
+    if (zip64ExtendedInfo.getOffsetLocalHeader() != -1) {
+      fileHeader.setOffsetLocalHeader(zip64ExtendedInfo.getOffsetLocalHeader());
+    }
+
+    if (zip64ExtendedInfo.getDiskNumberStart() != -1) {
+      fileHeader.setDiskNumberStart(zip64ExtendedInfo.getDiskNumberStart());
+    }
   }
 
   private void readZip64ExtendedInfo(LocalFileHeader localFileHeader, RawIO rawIO) throws ZipException {
@@ -432,18 +447,26 @@ public class HeaderReader {
       return;
     }
 
-    Zip64ExtendedInfo zip64ExtendedInfo = readZip64ExtendedInfo(localFileHeader.getExtraDataRecords(), rawIO);
+    Zip64ExtendedInfo zip64ExtendedInfo = readZip64ExtendedInfo(localFileHeader.getExtraDataRecords(), rawIO,
+        localFileHeader.getUncompressedSize(), localFileHeader.getCompressedSize(), 0);
 
     if (zip64ExtendedInfo == null) {
       return;
     }
 
     localFileHeader.setZip64ExtendedInfo(zip64ExtendedInfo);
-    localFileHeader.setUncompressedSize(zip64ExtendedInfo.getUncompressedSize());
-    localFileHeader.setCompressedSize(zip64ExtendedInfo.getCompressedSize());
+
+    if (zip64ExtendedInfo.getUncompressedSize() != -1) {
+      localFileHeader.setUncompressedSize(zip64ExtendedInfo.getUncompressedSize());
+    }
+
+    if (zip64ExtendedInfo.getCompressedSize() != -1) {
+      localFileHeader.setCompressedSize(zip64ExtendedInfo.getCompressedSize());
+    }
   }
 
-  private Zip64ExtendedInfo readZip64ExtendedInfo(List<ExtraDataRecord> extraDataRecords, RawIO rawIO)
+  private Zip64ExtendedInfo readZip64ExtendedInfo(List<ExtraDataRecord> extraDataRecords, RawIO rawIO,
+                                                  long uncompressedSize, long compressedSize, long offsetLocalHeader)
       throws ZipException {
 
     for (ExtraDataRecord extraDataRecord : extraDataRecords) {
@@ -461,17 +484,17 @@ public class HeaderReader {
         }
 
         int counter = 0;
-        if (counter < extraDataRecord.getSizeOfData()) {
+        if (counter < extraDataRecord.getSizeOfData() && uncompressedSize == ZIP_64_SIZE_LIMIT) {
           zip64ExtendedInfo.setUncompressedSize(rawIO.readLongLittleEndian(extraData, counter));
           counter += 8;
         }
 
-        if ( counter < extraDataRecord.getSizeOfData()) {
+        if ( counter < extraDataRecord.getSizeOfData() && compressedSize == ZIP_64_SIZE_LIMIT) {
           zip64ExtendedInfo.setCompressedSize(rawIO.readLongLittleEndian(extraData, counter));
           counter += 8;
         }
 
-        if (counter < extraDataRecord.getSizeOfData()) {
+        if (counter < extraDataRecord.getSizeOfData() && offsetLocalHeader == ZIP_64_SIZE_LIMIT) {
           zip64ExtendedInfo.setOffsetLocalHeader(rawIO.readLongLittleEndian(extraData, counter));
           counter += 8;
         }
@@ -504,7 +527,7 @@ public class HeaderReader {
     zip4jRaf.seek(zip4jRaf.getFilePointer() - 4 - 4 - 8 - 4 - 4);
   }
 
-  public LocalFileHeader readLocalFileHeader(InputStream inputStream) throws IOException {
+  public LocalFileHeader readLocalFileHeader(InputStream inputStream, Charset charset) throws IOException {
     LocalFileHeader localFileHeader = new LocalFileHeader();
     byte[] intBuff = new byte[4];
 
@@ -547,7 +570,7 @@ public class HeaderReader {
       // Modified after user reported an issue http://www.lingala.net/zip4j/forum/index.php?topic=2.0
 //				String fileName = new String(fileNameBuf, "Cp850");
 //				String fileName = Zip4jUtil.getCp850EncodedString(fileNameBuf);
-      String fileName = decodeStringWithCharset(fileNameBuf, localFileHeader.isFileNameUTF8Encoded());
+      String fileName = decodeStringWithCharset(fileNameBuf, localFileHeader.isFileNameUTF8Encoded(), charset);
 
       if (fileName == null) {
         throw new ZipException("file name is null, cannot assign file name to local file header");
