@@ -25,7 +25,6 @@ import net.lingala.zip4j.model.LocalFileHeader;
 import net.lingala.zip4j.model.Zip64EndOfCentralDirectoryLocator;
 import net.lingala.zip4j.model.Zip64EndOfCentralDirectoryRecord;
 import net.lingala.zip4j.model.ZipModel;
-import net.lingala.zip4j.util.BitUtils;
 import net.lingala.zip4j.util.InternalZipConstants;
 import net.lingala.zip4j.util.RawIO;
 
@@ -34,7 +33,6 @@ import java.io.File;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.nio.charset.Charset;
-import java.nio.charset.StandardCharsets;
 import java.util.List;
 
 import static net.lingala.zip4j.util.FileUtils.getZipFileNameWithoutExtension;
@@ -43,13 +41,15 @@ import static net.lingala.zip4j.util.Zip4jUtil.isStringNotNullAndNotEmpty;
 
 public class HeaderWriter {
 
-  private static final short ZIP64_EXTRA_DATA_RECORD_SIZE = 28;
+  private static final short ZIP64_EXTRA_DATA_RECORD_SIZE_LFH = 16;
+  private static final short ZIP64_EXTRA_DATA_RECORD_SIZE_FH = 28;
+  private static final short AES_EXTRA_DATA_RECORD_SIZE = 11;
 
   private RawIO rawIO = new RawIO();
   private byte[] longBuff = new byte[8];
 
-  public void writeLocalFileHeader(ZipModel zipModel, LocalFileHeader localFileHeader, OutputStream outputStream)
-      throws IOException {
+  public void writeLocalFileHeader(ZipModel zipModel, LocalFileHeader localFileHeader, OutputStream outputStream,
+                                   Charset charset) throws IOException {
 
     try(ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream()) {
       rawIO.writeIntLittleEndian(byteArrayOutputStream, (int) localFileHeader.getSignature().getValue());
@@ -62,9 +62,11 @@ public class HeaderWriter {
 
       rawIO.writeLongLittleEndian(longBuff, 0, localFileHeader.getCrc());
       byteArrayOutputStream.write(longBuff, 0, 4);
-      long uncompressedSize = localFileHeader.getUncompressedSize();
 
-      if (uncompressedSize >= ZIP_64_SIZE_LIMIT) {
+      boolean writeZip64Header = localFileHeader.getCompressedSize() >= ZIP_64_SIZE_LIMIT
+          || localFileHeader.getUncompressedSize() >= ZIP_64_SIZE_LIMIT;
+
+      if (writeZip64Header) {
         rawIO.writeLongLittleEndian(longBuff, 0, ZIP_64_SIZE_LIMIT);
 
         //Set the uncompressed size to ZipConstants.ZIP_64_SIZE_LIMIT as
@@ -81,26 +83,21 @@ public class HeaderWriter {
         rawIO.writeLongLittleEndian(longBuff, 0, localFileHeader.getUncompressedSize());
         byteArrayOutputStream.write(longBuff, 0, 4);
 
-        zipModel.setZip64Format(false);
         localFileHeader.setWriteCompressedSizeInZip64ExtraRecord(false);
       }
 
       byte[] fileNameBytes = new byte[0];
       if (isStringNotNullAndNotEmpty(localFileHeader.getFileName())) {
-        Charset charset = Charset.forName(InternalZipConstants.ZIP_STANDARD_CHARSET);
-        if (BitUtils.isBitSet(localFileHeader.getGeneralPurposeFlag()[1], 3)) {
-          charset = StandardCharsets.UTF_8;
-        }
         fileNameBytes = localFileHeader.getFileName().getBytes(charset);
       }
       rawIO.writeShortLittleEndian(byteArrayOutputStream, fileNameBytes.length);
 
       int extraFieldLength = 0;
       if (zipModel.isZip64Format()) {
-        extraFieldLength += 32;
+        extraFieldLength += ZIP64_EXTRA_DATA_RECORD_SIZE_LFH + 4; // 4 for signature + size of record
       }
       if (localFileHeader.getAesExtraDataRecord() != null) {
-        extraFieldLength += 11;
+        extraFieldLength += AES_EXTRA_DATA_RECORD_SIZE;
       }
       rawIO.writeShortLittleEndian(byteArrayOutputStream, extraFieldLength);
 
@@ -112,14 +109,12 @@ public class HeaderWriter {
       //This is NOT according to any specification but if this is changed
       //corresponding logic for updateLocalFileHeader for compressed size
       //has to be modified as well
-      if (zipModel.isZip64Format()) {
+      if (writeZip64Header) {
         rawIO.writeShortLittleEndian(byteArrayOutputStream,
             (int) HeaderSignature.ZIP64_EXTRA_FIELD_SIGNATURE.getValue());
-        rawIO.writeShortLittleEndian(byteArrayOutputStream, ZIP64_EXTRA_DATA_RECORD_SIZE);
+        rawIO.writeShortLittleEndian(byteArrayOutputStream, ZIP64_EXTRA_DATA_RECORD_SIZE_LFH);
         rawIO.writeLongLittleEndian(byteArrayOutputStream, localFileHeader.getUncompressedSize());
         rawIO.writeLongLittleEndian(byteArrayOutputStream, localFileHeader.getCompressedSize());
-        rawIO.writeLongLittleEndian(byteArrayOutputStream, 0); //Offset start of local file header
-        rawIO.writeIntLittleEndian(byteArrayOutputStream, 0); //Disk number start
       }
 
       if (localFileHeader.getAesExtraDataRecord() != null) {
@@ -168,7 +163,7 @@ public class HeaderWriter {
     }
   }
 
-  public void finalizeZipFile(ZipModel zipModel, OutputStream outputStream) throws IOException {
+  public void finalizeZipFile(ZipModel zipModel, OutputStream outputStream, Charset charset) throws IOException {
     if (zipModel == null || outputStream == null) {
       throw new ZipException("input parameters is null, cannot finalize zip file");
     }
@@ -176,7 +171,7 @@ public class HeaderWriter {
     try(ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream()) {
       processHeaderData(zipModel, outputStream);
       long offsetCentralDir = zipModel.getEndOfCentralDirectoryRecord().getOffsetOfStartOfCentralDirectory();
-      writeCentralDirectory(zipModel, byteArrayOutputStream, rawIO);
+      writeCentralDirectory(zipModel, byteArrayOutputStream, rawIO, charset);
       int sizeOfCentralDir = byteArrayOutputStream.size();
 
       if (zipModel.isZip64Format() || offsetCentralDir >= InternalZipConstants.ZIP_64_SIZE_LIMIT
@@ -208,11 +203,11 @@ public class HeaderWriter {
       }
 
       writeEndOfCentralDirectoryRecord(zipModel, sizeOfCentralDir, offsetCentralDir, byteArrayOutputStream, rawIO);
-      writeZipHeaderBytes(zipModel, outputStream, byteArrayOutputStream.toByteArray());
+      writeZipHeaderBytes(zipModel, outputStream, byteArrayOutputStream.toByteArray(), charset);
     }
   }
 
-  public void finalizeZipFileWithoutValidations(ZipModel zipModel, OutputStream outputStream) throws IOException {
+  public void finalizeZipFileWithoutValidations(ZipModel zipModel, OutputStream outputStream, Charset charset) throws IOException {
 
     if (zipModel == null || outputStream == null) {
       throw new ZipException("input parameters is null, cannot finalize zip file without validations");
@@ -220,7 +215,7 @@ public class HeaderWriter {
 
     try(ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream()) {
       long offsetCentralDir = zipModel.getEndOfCentralDirectoryRecord().getOffsetOfStartOfCentralDirectory();
-      writeCentralDirectory(zipModel, byteArrayOutputStream, rawIO);
+      writeCentralDirectory(zipModel, byteArrayOutputStream, rawIO, charset);
       int sizeOfCentralDir = byteArrayOutputStream.size();
 
       if (zipModel.isZip64Format() || offsetCentralDir >= InternalZipConstants.ZIP_64_SIZE_LIMIT
@@ -242,7 +237,7 @@ public class HeaderWriter {
       }
 
       writeEndOfCentralDirectoryRecord(zipModel, sizeOfCentralDir, offsetCentralDir, byteArrayOutputStream, rawIO);
-      writeZipHeaderBytes(zipModel, outputStream, byteArrayOutputStream.toByteArray());
+      writeZipHeaderBytes(zipModel, outputStream, byteArrayOutputStream.toByteArray(), charset);
     }
   }
 
@@ -301,7 +296,7 @@ public class HeaderWriter {
       //2 - size of zip64 data
       //8 - uncompressed size
       //8 - compressed size
-      int zip64CompressedSizeOffset = 2 + 2 + fileHeader.getFileNameLength() + 2 + 2 + 2;
+      int zip64CompressedSizeOffset = 2 + 2 + fileHeader.getFileNameLength() + 2 + 2;
       if (outputStream.skipBytes(zip64CompressedSizeOffset) != zip64CompressedSizeOffset) {
         throw new ZipException("Unable to skip " + zip64CompressedSizeOffset + " bytes to update LFH");
       }
@@ -333,7 +328,7 @@ public class HeaderWriter {
     return ((CountingOutputStream) outputStream).getCurrentSplitFileCounter();
   }
 
-  private void writeZipHeaderBytes(ZipModel zipModel, OutputStream outputStream, byte[] buff) throws IOException {
+  private void writeZipHeaderBytes(ZipModel zipModel, OutputStream outputStream, byte[] buff, Charset charset) throws IOException {
     if (buff == null) {
       throw new ZipException("invalid buff to write as zip headers");
     }
@@ -341,7 +336,7 @@ public class HeaderWriter {
     if (outputStream instanceof CountingOutputStream) {
       if (((CountingOutputStream) outputStream).checkBuffSizeAndStartNextSplitFile(buff.length)) {
         //TODO check if this is correct
-        finalizeZipFile(zipModel, outputStream);
+        finalizeZipFile(zipModel, outputStream, charset);
         return;
       }
     }
@@ -373,7 +368,7 @@ public class HeaderWriter {
     zipModel.getEndOfCentralDirectoryRecord().setNumberOfThisDiskStartOfCentralDir(currentSplitFileCounter);
   }
 
-  private void writeCentralDirectory(ZipModel zipModel, ByteArrayOutputStream byteArrayOutputStream, RawIO rawIO)
+  private void writeCentralDirectory(ZipModel zipModel, ByteArrayOutputStream byteArrayOutputStream, RawIO rawIO, Charset charset)
       throws ZipException {
 
     if (zipModel.getCentralDirectory() == null || zipModel.getCentralDirectory().getFileHeaders() == null
@@ -382,12 +377,12 @@ public class HeaderWriter {
     }
 
     for (FileHeader fileHeader: zipModel.getCentralDirectory().getFileHeaders()) {
-      writeFileHeader(zipModel, fileHeader, byteArrayOutputStream, rawIO);
+      writeFileHeader(zipModel, fileHeader, byteArrayOutputStream, rawIO, charset);
     }
   }
 
   private void writeFileHeader(ZipModel zipModel, FileHeader fileHeader, ByteArrayOutputStream byteArrayOutputStream,
-                              RawIO rawIO) throws ZipException {
+                              RawIO rawIO, Charset charset) throws ZipException {
     if (fileHeader == null) {
       throw new ZipException("input parameters is null, cannot write local file header");
     }
@@ -396,6 +391,10 @@ public class HeaderWriter {
       final byte[] emptyShortByte = {0, 0};
       boolean writeZip64ExtendedInfo = false;
 
+      if (isZip64Entry(fileHeader)) {
+        writeZip64ExtendedInfo = true;
+      }
+
       rawIO.writeIntLittleEndian(byteArrayOutputStream, (int) fileHeader.getSignature().getValue());
       rawIO.writeShortLittleEndian(byteArrayOutputStream, fileHeader.getVersionMadeBy());
       rawIO.writeShortLittleEndian(byteArrayOutputStream, fileHeader.getVersionNeededToExtract());
@@ -408,11 +407,11 @@ public class HeaderWriter {
       rawIO.writeLongLittleEndian(longBuff, 0, fileHeader.getCrc());
       byteArrayOutputStream.write(longBuff, 0, 4);
 
-      if (fileHeader.getCompressedSize() >= ZIP_64_SIZE_LIMIT || fileHeader.getUncompressedSize() >= ZIP_64_SIZE_LIMIT) {
+      if (writeZip64ExtendedInfo) {
         rawIO.writeLongLittleEndian(longBuff, 0, ZIP_64_SIZE_LIMIT);
         byteArrayOutputStream.write(longBuff, 0, 4);
         byteArrayOutputStream.write(longBuff, 0, 4);
-        writeZip64ExtendedInfo = true;
+        zipModel.setZip64Format(true);
       } else {
         rawIO.writeLongLittleEndian(longBuff, 0, fileHeader.getCompressedSize());
         byteArrayOutputStream.write(longBuff, 0, 4);
@@ -422,10 +421,6 @@ public class HeaderWriter {
 
       byte[] fileNameBytes = new byte[0];
       if (isStringNotNullAndNotEmpty(fileHeader.getFileName())) {
-        Charset charset = Charset.forName(InternalZipConstants.ZIP_STANDARD_CHARSET);
-        if (BitUtils.isBitSet(fileHeader.getGeneralPurposeFlag()[1], 3)) {
-          charset = StandardCharsets.UTF_8;
-        }
         fileNameBytes = fileHeader.getFileName().getBytes(charset);
       }
       rawIO.writeShortLittleEndian(byteArrayOutputStream, fileNameBytes.length);
@@ -433,10 +428,9 @@ public class HeaderWriter {
       //Compute offset bytes before extra field is written for Zip64 compatibility
       //NOTE: this data is not written now, but written at a later point
       byte[] offsetLocalHeaderBytes = new byte[4];
-      if (fileHeader.getOffsetLocalHeader() > ZIP_64_SIZE_LIMIT) {
+      if (writeZip64ExtendedInfo) {
         rawIO.writeLongLittleEndian(longBuff, 0, ZIP_64_SIZE_LIMIT);
         System.arraycopy(longBuff, 0, offsetLocalHeaderBytes, 0, 4);
-        writeZip64ExtendedInfo = true;
       } else {
         rawIO.writeLongLittleEndian(longBuff, 0, fileHeader.getOffsetLocalHeader());
         System.arraycopy(longBuff, 0, offsetLocalHeaderBytes, 0, 4);
@@ -444,20 +438,16 @@ public class HeaderWriter {
 
       int extraFieldLength = 0;
       if (writeZip64ExtendedInfo) {
-        extraFieldLength += 32;
+        extraFieldLength += ZIP64_EXTRA_DATA_RECORD_SIZE_FH + 4; // 4 for signature + size of record
       }
       if (fileHeader.getAesExtraDataRecord() != null) {
-        extraFieldLength += 11;
+        extraFieldLength += AES_EXTRA_DATA_RECORD_SIZE;
       }
       rawIO.writeShortLittleEndian(byteArrayOutputStream, extraFieldLength);
 
       String fileComment = fileHeader.getFileComment();
       byte[] fileCommentBytes = new byte[0];
       if (isStringNotNullAndNotEmpty(fileComment)) {
-        Charset charset = Charset.forName(InternalZipConstants.ZIP_STANDARD_CHARSET);
-        if (BitUtils.isBitSet(fileHeader.getGeneralPurposeFlag()[1], 3)) {
-          charset = StandardCharsets.UTF_8;
-        }
         fileCommentBytes = fileComment.getBytes(charset);
       }
       rawIO.writeShortLittleEndian(byteArrayOutputStream, fileCommentBytes.length);
@@ -486,7 +476,7 @@ public class HeaderWriter {
             (int) HeaderSignature.ZIP64_EXTRA_FIELD_SIGNATURE.getValue());
 
         //size of data
-        rawIO.writeShortLittleEndian(byteArrayOutputStream, ZIP64_EXTRA_DATA_RECORD_SIZE);
+        rawIO.writeShortLittleEndian(byteArrayOutputStream, ZIP64_EXTRA_DATA_RECORD_SIZE_FH);
         rawIO.writeLongLittleEndian(byteArrayOutputStream, fileHeader.getUncompressedSize());
         rawIO.writeLongLittleEndian(byteArrayOutputStream, fileHeader.getCompressedSize());
         rawIO.writeLongLittleEndian(byteArrayOutputStream, fileHeader.getOffsetLocalHeader());
@@ -607,7 +597,7 @@ public class HeaderWriter {
 
     String comment = zipModel.getEndOfCentralDirectoryRecord().getComment();
     if (isStringNotNullAndNotEmpty(comment)) {
-      byte[] commentBytes = comment.getBytes(StandardCharsets.UTF_8);
+      byte[] commentBytes = comment.getBytes(InternalZipConstants.CHARSET_UTF_8);
       rawIO.writeShortLittleEndian(byteArrayOutputStream, commentBytes.length);
       byteArrayOutputStream.write(commentBytes);
     } else {
@@ -629,4 +619,9 @@ public class HeaderWriter {
     return noEntries;
   }
 
+  private boolean isZip64Entry(FileHeader fileHeader) {
+    return fileHeader.getCompressedSize() >= ZIP_64_SIZE_LIMIT
+        || fileHeader.getUncompressedSize() >= ZIP_64_SIZE_LIMIT
+        || fileHeader.getOffsetLocalHeader() >= ZIP_64_SIZE_LIMIT;
+  }
 }
\ No newline at end of file
