@@ -69,6 +69,20 @@ public class AddFilesToZipIT extends AbstractIT {
         EncryptionMethod.ZIP_STANDARD, null);
   }
 
+  @Test
+  public void testAddFileAsStringWithZipParametersStoreAndStandardEncryptionAndCharsetCp949() throws IOException {
+    ZipParameters zipParameters = createZipParameters(EncryptionMethod.ZIP_STANDARD, null);
+    zipParameters.setCompressionMethod(CompressionMethod.STORE);
+    ZipFile zipFile = new ZipFile(generatedZipFile, PASSWORD);
+
+    zipFile.setCharset(CHARSET_CP_949);
+    String koreanFileName = "가나다.abc";
+    zipFile.addFile(TestUtils.getTestFileFromResources(koreanFileName).getPath(), zipParameters);
+
+    ZipFileVerifier.verifyZipFileByExtractingAllFiles(generatedZipFile, PASSWORD, outputFolder, 1, true, CHARSET_CP_949);
+    assertThat(zipFile.getFileHeaders().get(0).getFileName()).isEqualTo(koreanFileName);
+  }
+
   @Test
   public void testAddFileThrowsExceptionWhenFileDoesNotExist() throws ZipException {
     expectedException.expectMessage("File does not exist: somefile.txt");
@@ -194,7 +208,7 @@ public class AddFilesToZipIT extends AbstractIT {
     assertThat(zipFile.getFileHeaders()).hasSize(1);
     assertThat(zipFile.getFileHeader("/data/newfile.txt")).isNotNull();
     assertThat(zipFile.getFileHeader("sample_text_large.txt")).isNull();
-    zipFile.extractAll(outputFolder.getCanonicalPath());
+    zipFile.extractAll(outputFolder.getPath());
   }
 
   @Test
@@ -557,6 +571,24 @@ public class AddFilesToZipIT extends AbstractIT {
     ZipFileVerifier.verifyZipFileByExtractingAllFiles(generatedZipFile, PASSWORD, outputFolder, 13);
   }
 
+  @Test
+  public void testAddFolderWithNotNormalizedPath() throws IOException {
+    ZipFile zipFile = new ZipFile(generatedZipFile);
+    ZipParameters parameters = new ZipParameters();
+
+    String folderToAddPath = TestUtils.getTestFileFromResources("").getPath()
+        + InternalZipConstants.FILE_SEPARATOR + ".."
+        + InternalZipConstants.FILE_SEPARATOR
+        + TestUtils.getTestFileFromResources("").getName();
+    File folderToAdd = new File(folderToAddPath);
+    zipFile.addFolder(folderToAdd, parameters);
+
+    File fileToAdd = TestUtils.getTestFileFromResources("file_PDF_1MB.pdf");
+    zipFile.addFile(fileToAdd, parameters);
+
+    ZipFileVerifier.verifyZipFileByExtractingAllFiles(generatedZipFile, outputFolder, 13);
+  }
+
   @Test
   public void testAddStreamToZipThrowsExceptionWhenFileNameIsNull() throws IOException {
     ZipFile zipFile = new ZipFile(generatedZipFile);
@@ -564,7 +596,7 @@ public class AddFilesToZipIT extends AbstractIT {
     ZipParameters zipParameters = new ZipParameters();
     zipParameters.setFileNameInZip(null);
 
-    expectedException.expectMessage("fileNameInZip is null or empty");
+    expectedException.expectMessage("fileNameInZip has to be set in zipParameters when adding stream");
     expectedException.expect(ZipException.class);
 
     zipFile.addStream(inputStream, zipParameters);
@@ -577,7 +609,7 @@ public class AddFilesToZipIT extends AbstractIT {
     ZipParameters zipParameters = new ZipParameters();
     zipParameters.setFileNameInZip("");
 
-    expectedException.expectMessage("fileNameInZip is null or empty");
+    expectedException.expectMessage("fileNameInZip has to be set in zipParameters when adding stream");
     expectedException.expect(ZipException.class);
 
     zipFile.addStream(inputStream, zipParameters);
@@ -638,6 +670,43 @@ public class AddFilesToZipIT extends AbstractIT {
     assertThat(BitUtils.isBitSet(generalPurposeBytes[0], 3)).isTrue();
   }
 
+  @Test
+  public void testAddStreamWithStoreCompressionAndCharset() throws IOException {
+    String koreanFileName = "가나다.abc";
+    File fileToAdd = TestUtils.getTestFileFromResources(koreanFileName);
+    ZipParameters zipParameters = new ZipParameters();
+    zipParameters.setCompressionMethod(CompressionMethod.STORE);
+    zipParameters.setFileNameInZip(fileToAdd.getName());
+    ZipFile zipFile = new ZipFile(generatedZipFile);
+    InputStream inputStream = new FileInputStream(fileToAdd);
+
+    zipFile.setCharset(CHARSET_CP_949);
+    zipFile.addStream(inputStream, zipParameters);
+
+    byte[] generalPurposeBytes = zipFile.getFileHeaders().get(0).getGeneralPurposeFlag();
+    // assert that extra data record is not present
+    assertThat(BitUtils.isBitSet(generalPurposeBytes[1], 3)).isFalse();
+    assertThat(zipFile.getFileHeaders().get(0).getFileName()).isEqualTo(koreanFileName);
+  }
+
+  @Test
+  public void testAddStreamWithStoreCompressionAndDefaultCharset() throws IOException {
+    String koreanFileName = "가나다.abc";
+    File fileToAdd = TestUtils.getTestFileFromResources(koreanFileName);
+    ZipParameters zipParameters = new ZipParameters();
+    zipParameters.setCompressionMethod(CompressionMethod.STORE);
+    zipParameters.setFileNameInZip(fileToAdd.getName());
+    ZipFile zipFile = new ZipFile(generatedZipFile);
+    InputStream inputStream = new FileInputStream(fileToAdd);
+
+    zipFile.addStream(inputStream, zipParameters);
+
+    byte[] generalPurposeBytes = zipFile.getFileHeaders().get(0).getGeneralPurposeFlag();
+    // assert that extra data record is not present
+    assertThat(BitUtils.isBitSet(generalPurposeBytes[1], 3)).isTrue();
+    assertThat(zipFile.getFileHeaders().get(0).getFileName()).isEqualTo(koreanFileName);
+  }
+
   @Test
   public void testAddStreamToZipWithAesEncryptionForNewZipAddsSuccessfully() throws IOException {
     File fileToAdd = TestUtils.getTestFileFromResources("бореиская.txt");
@@ -701,6 +770,37 @@ public class AddFilesToZipIT extends AbstractIT {
         AesKeyStrength.KEY_STRENGTH_256, AesVersion.ONE);
   }
 
+  @Test
+  public void testAddStreamToZipWithCharsetCp949() throws IOException {
+    String koreanFileName = "가나다.abc";
+    ZipFile zipFile = new ZipFile(generatedZipFile);
+    File fileToAdd = TestUtils.getTestFileFromResources(koreanFileName);
+    InputStream inputStream = new FileInputStream(fileToAdd);
+    ZipParameters zipParameters = new ZipParameters();
+
+    zipParameters.setFileNameInZip(fileToAdd.getName());
+    zipFile.setCharset(CHARSET_CP_949);
+    zipFile.addStream(inputStream, zipParameters);
+
+    ZipFileVerifier.verifyZipFileByExtractingAllFiles(generatedZipFile, null, outputFolder, 1, true, CHARSET_CP_949);
+    assertThat(zipFile.getFileHeaders().get(0).getFileName()).isEqualTo(koreanFileName);
+  }
+
+  @Test
+  public void testAddStreamToZipWithSameEntryNameRemovesOldEntry() throws IOException {
+    File fileToAdd = TestUtils.getTestFileFromResources("sample.pdf");
+    ZipFile zipFile = new ZipFile(generatedZipFile);
+    zipFile.addFile(fileToAdd);
+
+    try(InputStream inputStream = new FileInputStream(fileToAdd)) {
+      ZipParameters zipParameters = new ZipParameters();
+      zipParameters.setFileNameInZip("sample.pdf");
+      zipFile.addStream(inputStream, zipParameters);
+    }
+
+    ZipFileVerifier.verifyZipFileByExtractingAllFiles(generatedZipFile, outputFolder, 1);
+  }
+
   private void verifyZipFileContainsFiles(File generatedZipFile, List<String> fileNames,
                                           CompressionMethod compressionMethod, EncryptionMethod encryptionMethod,
                                           AesKeyStrength aesKeyStrength) throws ZipException {
